#!/usr/bin/env python3
# This amulet test deploys the bundles.yaml file in this directory.

import os
import unittest
import yaml
import amulet
import sys

# Lots of prereqs on this charm (eg: java), so give it a large timeout
seconds_to_wait = 1500


class BundleTest(unittest.TestCase):
    """ Create a class for testing the charm in the unit test framework. """
    @classmethod
    def setUpClass(cls):
        """ Set up an amulet deployment using the bundle. """
        d = amulet.Deployment(juju_env='local', series='trusty')
        local_path = os.path.join(os.path.dirname(__file__), 'local.yaml')
        with open(local_path, "r") as fd:
            config = yaml.safe_load(fd)
        curl_url = config.get('mq').get('mq_curl_url')
        print('Using mq_curl_url  %s' % curl_url)
        # Test if a IBM mq curl url is defined
        if not curl_url:
            print("You need to define a im_curl_url.\n"
                  "Edit local.yaml or tests/00-setup and run it again.")
            sys.exit(1)

        curl_opts = config.get('mq').get('mq_curl_opts')
        print('Using mq_curl_opts  %s' % curl_opts)
        # Test if IBM  curl_opts is defined
        if not curl_opts:
            print("You need to define a curl_opts."
                  " repository.\n Edit local.yaml or tests/00-setup"
                  " and run it again.")
            sys.exit(1)
        license_accepted = config.get('mq').get('mq_license_accepted')
        print('Using license_accepted  %s' % license_accepted)
        # Test if a license_accepted is defined
        if not license_accepted:
            print("You need to define a license_accepted for the IBM MQ"
                  " product repository.\n Edit local.yaml or tests/00-setup"
                  " and run it again.")
            sys.exit(1)

        d.add('ibm-mq')

        # Software doesn't actually install until you accept the license
        d.configure('ibm-mq', {'license_accepted': license_accepted,
                               'curl_url': curl_url,
                               'curl_opts': curl_opts})

        d.setup(seconds_to_wait)
        d.sentry.wait(seconds_to_wait)
        cls.d = d

    def test_deployed(self):
        """ Test to see if the bundle deployed successfully. """
        self.assertTrue(self.d.deployed)

    def test_running(self):
        """ Test that, if deployed,
            everything is set up and running correctly. """
        ibm_mq_unit = self.d.sentry.unit['ibm-mq'][0]

        # Configure system values
        output, code = ibm_mq_unit.run('sysctl kernel.shmmax | cut -d= -f2')
        print(output)
        if code != 0 or int(output) < 268435456:
            # this isn't really a pass/fail test, as it depends
            # on the environment to which
            # the charm is deployed, and doesn't block the install
            print('Unable to set kernel.shmmax'
                  ' value (perhaps on local provider?).')

        output, code = ibm_mq_unit.run('sysctl fs.file-max | cut -d= -f2')
        print(output)
        if code != 0 or int(output) < 524288:
            # this isn't really a pass/fail test, as it depends
            # on the environment to which
            # the charm is deployed, and doesn't block the install
            print('Unable to set fs.file value (perhaps on local provider?).')

        output, code = ibm_mq_unit.run('sysctl -p')
        print(output)
        assert code == 0, 'Unable to configure system value.'

        # Verify that the ubuntu user is in the mqm group
        output, code = ibm_mq_unit.run('groups ubuntu')
        print(output)
        assert 'mqm' in output, 'Ubuntu user not in mqm group.'

        # Running as mqm, root does not work
        # Add Queue Manager QMA
        cmd = 'su -l mqm -c "/opt/mqm/bin/crtmqm QMA"'
        output, code = ibm_mq_unit.run(cmd)
        print(output)
        assert code == 0, 'Unable to create the MQ queue manager QMA'

        # Start the queue manager
        cmd = 'su -l mqm -c "/opt/mqm/bin/strmqm QMA"'
        output, code = ibm_mq_unit.run(cmd)
        print(output)
        assert code == 0, 'Unable to start the MQ queue manager'

if __name__ == '__main__':
    unittest.main()
